#!/bin/bash

KOMPIRA_REPO_DIR=/opt/kompira/repo
MOUNT_DEVICE=/dev/dvd
SETUP_LOG=pkg_setup.$$.log
THIS_DIR=`dirname \`readlink -f $0\``
SYSTEM="CentOS"
MEDIA_REPO=c6-media

start_setup()
{
    echo "***"
    echo "*** Kompira に必要な外部パッケージをインストールします"
    echo "***"
}

exit_setup()
{
    echo "***"
    echo "*** 外部パッケージのインストールが完了しました"
    echo "*** 引き続きオフラインモードで Kompira のインストールを行ってください"
    echo "*** (install.sh --offline)"
    echo "***"
}

exit_if_failed()
{
    local status message
    status="$1"
    message="$2"
    if [ "$status" -ne 0 ]
    then
        echo
        echo -e "*** [ERROR] ${message}"
        echo
        exit 1
    fi
}

is_yum_installed()
{
    for pkg
    do
        if ! yum list installed $pkg > /dev/null 2>&1
        then
            return 1
        fi
    done
}

mount_dvd()
{
    echo
    echo "*** DVD ドライブをマウントします"
    echo
    mkdir -p /media/${SYSTEM}
    mount -t iso9660 $MOUNT_DEVICE /media/${SYSTEM}
    exit_if_failed "$?" "DVD ドライブのマウントに失敗しました\n(${SYSTEM} インストール DVD がセットされていることを確認してください)"

    if [ $SYSTEM == "RHEL" ]
    then
        MEDIA_REPO=InstallMedia
        cp /media/${SYSTEM}/media.repo /etc/yum.repos.d/media.repo
        cat <<EOF >> /etc/yum.repos.d/media.repo
baseurl=file:///media/${SYSTEM}/
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release
EOF
    fi
}

copy_kompira_repo()
{
    echo
    echo "*** Kompira 用レポジトリをコピーします"
    echo
    mkdir -p /opt/kompira
    cp -r $THIS_DIR/repo $KOMPIRA_REPO_DIR
    exit_if_failed "$?" "Kompira 用レポジトリのコピーに失敗しました"
}

install_utils()
{
    echo
    echo "*** セットアップ用パッケージをインストールします"
    echo
    local pkg
    for pkg in yum-utils python-setuptools
    do
        yum --disablerepo=* --enablerepo=${MEDIA_REPO} install -y $pkg
        exit_if_failed "$?" "$pkg のインストールに失敗しました"
    done

}

add_repository()
{
    echo
    echo "*** 拡張レポジトリを追加します"
    echo
    local pkg
    local repo
    for pkg in epel-release-6-*.noarch.rpm pgdg-redhat93-9.3-*.noarch.rpm
    do
        rpm -i $THIS_DIR/$pkg
        exit_if_failed "$?" "$pkg のインストールに失敗しました"
    done
    for repo in Kompira.repo
    do
        yum-config-manager --disablerepo=* --enablerepo=${MEDIA_REPO} --add-repo=$THIS_DIR/$repo
        exit_if_failed "$?" "レポジトリ $repo の追加に失敗しました"
    done
}

config_repository()
{
    echo
    echo "*** レポジトリの設定を行います"
    echo
    local repo
    local repos
    if [ $SYSTEM == "CentOS" ]
    then
        repos="base extras updates epel pgdg93"
    else
        repos="rhel-6-server-rpms rhel-6-server-optional-rpms epel pgdg93"
    fi
    for repo in $repos
    do
        yum-config-manager --disable $repo
        exit_if_failed "$?" "$repo の無効化に失敗しました"
    done
    for repo in ${MEDIA_REPO}
    do
        yum-config-manager --enable $repo
        exit_if_failed "$?" "$repo の有効化に失敗しました"
    done
}

install_yum_pkgs()
{
    echo
    echo "*** yum パッケージをインストールします"
    echo
    local pkg
    for pkg in rabbitmq-server postgresql93-server postgresql93-contrib postgresql93-devel
    do
        yum install -y $pkg
        exit_if_failed "$?" "$pkg のインストールに失敗しました"
    done
}

install_pip()
{
    echo
    echo "*** pip をインストールします"
    echo
    python $THIS_DIR/pip/get-pip.py --no-index --find-links=$THIS_DIR/pip --no-wheel
    exit_if_failed "$?" "pip のインストールに失敗しました"

    pip install wheel --find-links=wheelhouse/ --no-index --no-deps
    exit_if_failed "$?" "wheel のインストールに失敗しました"
}

install_python_pkgs()
{
    echo
    echo "*** Python パッケージをインストールします"
    echo
    for whl_pkg in $THIS_DIR/wheelhouse/*
    do
        if [[ $whl_pkg =~ /wheelhouse/pyOpenSSL-.* ]] && is_yum_installed pyOpenSSL
        then
            pip install -U --target=/usr/lib64/python2.6/site-packages --use-wheel --no-index --no-deps --find-links=$THIS_DIR/wheelhouse/ $whl_pkg
        else
            pip install --use-wheel --no-index --no-deps --find-links=$THIS_DIR/wheelhouse/ $whl_pkg
        fi
    done
    exit_if_failed "$?" "Python パッケージのインストールに失敗しました"
}

################################################################

{
start_setup
mount_dvd
copy_kompira_repo
install_utils
add_repository
config_repository
install_yum_pkgs
install_pip
install_python_pkgs
exit_setup
} 2>&1 | tee $SETUP_LOG
